@@ -26,7 +26,7 @@ import jakarta.annotation.Nullable;
  */
 public interface PolarisStorageIntegrationProvider {
   @SuppressWarnings("unchecked")
-  <T extends PolarisStorageConfigurationInfo> @Nullable
-      PolarisStorageIntegration<T> getStorageIntegrationForConfig(
+  <T extends PolarisStorageConfigurationInfo>
+      @Nullable PolarisStorageIntegration<T> getStorageIntegrationForConfig(
           PolarisStorageConfigurationInfo polarisStorageConfigurationInfo);
 }
