@@ -37,12 +37,13 @@ import java.util.stream.Stream;
 import org.apache.polaris.core.PolarisCallContext;
 import org.apache.polaris.core.context.CallContext;
 import org.apache.polaris.core.entity.AsyncTaskType;
+import org.apache.polaris.core.entity.EntityNameLookupRecord;
 import org.apache.polaris.core.entity.PolarisBaseEntity;
 import org.apache.polaris.core.entity.PolarisEntity;
-import org.apache.polaris.core.entity.PolarisEntityActiveRecord;
 import org.apache.polaris.core.entity.PolarisEntitySubType;
 import org.apache.polaris.core.entity.PolarisEntityType;
 import org.apache.polaris.core.entity.TaskEntity;
+import org.apache.polaris.core.persistence.pagination.PageToken;
 import org.assertj.core.api.Assertions;
 import org.assertj.core.api.InstanceOfAssertFactories;
 import org.junit.jupiter.api.BeforeEach;
@@ -76,163 +77,147 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
   /** validate that the root catalog was properly constructed */
   @Test
-  void validateBootstrap() {
+  protected void validateBootstrap() {
     // allocate test driver
     polarisTestMetaStoreManager.validateBootstrap();
   }
 
   @Test
-  void testCreateTestCatalog() {
+  protected void testCreateTestCatalog() {
     // allocate test driver
     polarisTestMetaStoreManager.testCreateTestCatalog();
   }
 
   @Test
-  void testCreateTestCatalogWithRetry() {
+  protected void testCreateTestCatalogWithRetry() {
     // allocate test driver
     polarisTestMetaStoreManager.forceRetry();
     polarisTestMetaStoreManager.testCreateTestCatalog();
   }
 
   @Test
-  void testBrowse() {
+  protected void testBrowse() {
     // allocate test driver
     polarisTestMetaStoreManager.testBrowse();
   }
 
   @Test
-  void testCreateEntities() {
+  protected void testCreateEntities() {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
-    try (CallContext callCtx =
-        CallContext.of(() -> "testRealm", polarisTestMetaStoreManager.polarisCallContext)) {
-      if (CallContext.getCurrentContext() == null) {
-        CallContext.setCurrentContext(callCtx);
-      }
-      TaskEntity task1 = createTask("task1", 100L);
-      TaskEntity task2 = createTask("task2", 101L);
-      List<PolarisBaseEntity> createdEntities =
-          metaStoreManager
-              .createEntitiesIfNotExist(
-                  polarisTestMetaStoreManager.polarisCallContext, null, List.of(task1, task2))
-              .getEntities();
-
-      Assertions.assertThat(createdEntities)
-          .isNotNull()
-          .hasSize(2)
-          .extracting(PolarisEntity::toCore)
-          .containsExactly(PolarisEntity.toCore(task1), PolarisEntity.toCore(task2));
-
-      List<PolarisEntityActiveRecord> listedEntities =
-          metaStoreManager
-              .listEntities(
-                  polarisTestMetaStoreManager.polarisCallContext,
-                  null,
-                  PolarisEntityType.TASK,
-                  PolarisEntitySubType.NULL_SUBTYPE)
-              .getEntities();
-      Assertions.assertThat(listedEntities)
-          .isNotNull()
-          .hasSize(2)
-          .containsExactly(
-              new PolarisEntityActiveRecord(
-                  task1.getCatalogId(),
-                  task1.getId(),
-                  task1.getParentId(),
-                  task1.getName(),
-                  task1.getTypeCode(),
-                  task1.getSubTypeCode()),
-              new PolarisEntityActiveRecord(
-                  task2.getCatalogId(),
-                  task2.getId(),
-                  task2.getParentId(),
-                  task2.getName(),
-                  task2.getTypeCode(),
-                  task2.getSubTypeCode()));
+    if (CallContext.getCurrentContext() == null) {
+      CallContext.setCurrentContext(polarisTestMetaStoreManager.polarisCallContext);
     }
+    TaskEntity task1 = createTask("task1", 100L);
+    TaskEntity task2 = createTask("task2", 101L);
+    List<PolarisBaseEntity> createdEntities =
+        metaStoreManager
+            .createEntitiesIfNotExist(
+                polarisTestMetaStoreManager.polarisCallContext, null, List.of(task1, task2))
+            .getEntities();
+
+    Assertions.assertThat(createdEntities)
+        .isNotNull()
+        .hasSize(2)
+        .extracting(PolarisEntity::toCore)
+        .containsExactly(PolarisEntity.toCore(task1), PolarisEntity.toCore(task2));
+
+    List<EntityNameLookupRecord> listedEntities =
+        metaStoreManager
+            .listEntities(
+                polarisTestMetaStoreManager.polarisCallContext,
+                null,
+                PolarisEntityType.TASK,
+                PolarisEntitySubType.NULL_SUBTYPE,
+                PageToken.readEverything())
+            .getEntities();
+    Assertions.assertThat(listedEntities)
+        .isNotNull()
+        .hasSize(2)
+        .containsExactly(
+            new EntityNameLookupRecord(
+                task1.getCatalogId(),
+                task1.getId(),
+                task1.getParentId(),
+                task1.getName(),
+                task1.getTypeCode(),
+                task1.getSubTypeCode()),
+            new EntityNameLookupRecord(
+                task2.getCatalogId(),
+                task2.getId(),
+                task2.getParentId(),
+                task2.getName(),
+                task2.getTypeCode(),
+                task2.getSubTypeCode()));
   }
 
   @Test
-  void testCreateEntitiesAlreadyExisting() {
+  protected void testCreateEntitiesAlreadyExisting() {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
-    try (CallContext callCtx =
-        CallContext.of(() -> "testRealm", polarisTestMetaStoreManager.polarisCallContext)) {
-      if (CallContext.getCurrentContext() == null) {
-        CallContext.setCurrentContext(callCtx);
-      }
-      TaskEntity task1 = createTask("task1", 100L);
-      TaskEntity task2 = createTask("task2", 101L);
-      List<PolarisBaseEntity> createdEntities =
-          metaStoreManager
-              .createEntitiesIfNotExist(
-                  polarisTestMetaStoreManager.polarisCallContext, null, List.of(task1, task2))
-              .getEntities();
-
-      Assertions.assertThat(createdEntities)
-          .isNotNull()
-          .hasSize(2)
-          .extracting(PolarisEntity::toCore)
-          .containsExactly(PolarisEntity.toCore(task1), PolarisEntity.toCore(task2));
-
-      TaskEntity task3 = createTask("task3", 103L);
-
-      // entities task1 and task2 already exist with the same identifier, so the full list is
-      // returned
-      createdEntities =
-          metaStoreManager
-              .createEntitiesIfNotExist(
-                  polarisTestMetaStoreManager.polarisCallContext,
-                  null,
-                  List.of(task1, task2, task3))
-              .getEntities();
-      Assertions.assertThat(createdEntities)
-          .isNotNull()
-          .hasSize(3)
-          .extracting(PolarisEntity::toCore)
-          .containsExactly(
-              PolarisEntity.toCore(task1),
-              PolarisEntity.toCore(task2),
-              PolarisEntity.toCore(task3));
+    if (CallContext.getCurrentContext() == null) {
+      CallContext.setCurrentContext(polarisTestMetaStoreManager.polarisCallContext);
     }
+    TaskEntity task1 = createTask("task1", 100L);
+    TaskEntity task2 = createTask("task2", 101L);
+    List<PolarisBaseEntity> createdEntities =
+        metaStoreManager
+            .createEntitiesIfNotExist(
+                polarisTestMetaStoreManager.polarisCallContext, null, List.of(task1, task2))
+            .getEntities();
+
+    Assertions.assertThat(createdEntities)
+        .isNotNull()
+        .hasSize(2)
+        .extracting(PolarisEntity::toCore)
+        .containsExactly(PolarisEntity.toCore(task1), PolarisEntity.toCore(task2));
+
+    TaskEntity task3 = createTask("task3", 103L);
+
+    // entities task1 and task2 already exist with the same identifier, so the full list is
+    // returned
+    createdEntities =
+        metaStoreManager
+            .createEntitiesIfNotExist(
+                polarisTestMetaStoreManager.polarisCallContext, null, List.of(task1, task2, task3))
+            .getEntities();
+    Assertions.assertThat(createdEntities)
+        .isNotNull()
+        .hasSize(3)
+        .extracting(PolarisEntity::toCore)
+        .containsExactly(
+            PolarisEntity.toCore(task1), PolarisEntity.toCore(task2), PolarisEntity.toCore(task3));
   }
 
   @Test
-  void testCreateEntitiesWithConflict() {
+  protected void testCreateEntitiesWithConflict() {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
-    try (CallContext callCtx =
-        CallContext.of(() -> "testRealm", polarisTestMetaStoreManager.polarisCallContext)) {
-      if (CallContext.getCurrentContext() == null) {
-        CallContext.setCurrentContext(callCtx);
-      }
-      TaskEntity task1 = createTask("task1", 100L);
-      TaskEntity task2 = createTask("task2", 101L);
-      TaskEntity task3 = createTask("task3", 103L);
-      List<PolarisBaseEntity> createdEntities =
-          metaStoreManager
-              .createEntitiesIfNotExist(
-                  polarisTestMetaStoreManager.polarisCallContext,
-                  null,
-                  List.of(task1, task2, task3))
-              .getEntities();
-
-      Assertions.assertThat(createdEntities)
-          .isNotNull()
-          .hasSize(3)
-          .extracting(PolarisEntity::toCore)
-          .containsExactly(
-              PolarisEntity.toCore(task1),
-              PolarisEntity.toCore(task2),
-              PolarisEntity.toCore(task3));
-
-      TaskEntity secondTask3 = createTask("task3", 104L);
-
-      TaskEntity task4 = createTask("task4", 105L);
-      createdEntities =
-          metaStoreManager
-              .createEntitiesIfNotExist(
-                  polarisTestMetaStoreManager.polarisCallContext, null, List.of(secondTask3, task4))
-              .getEntities();
-      Assertions.assertThat(createdEntities).isNull();
+    if (CallContext.getCurrentContext() == null) {
+      CallContext.setCurrentContext(polarisTestMetaStoreManager.polarisCallContext);
     }
+    TaskEntity task1 = createTask("task1", 100L);
+    TaskEntity task2 = createTask("task2", 101L);
+    TaskEntity task3 = createTask("task3", 103L);
+    List<PolarisBaseEntity> createdEntities =
+        metaStoreManager
+            .createEntitiesIfNotExist(
+                polarisTestMetaStoreManager.polarisCallContext, null, List.of(task1, task2, task3))
+            .getEntities();
+
+    Assertions.assertThat(createdEntities)
+        .isNotNull()
+        .hasSize(3)
+        .extracting(PolarisEntity::toCore)
+        .containsExactly(
+            PolarisEntity.toCore(task1), PolarisEntity.toCore(task2), PolarisEntity.toCore(task3));
+
+    TaskEntity secondTask3 = createTask("task3", 104L);
+
+    TaskEntity task4 = createTask("task4", 105L);
+    createdEntities =
+        metaStoreManager
+            .createEntitiesIfNotExist(
+                polarisTestMetaStoreManager.polarisCallContext, null, List.of(secondTask3, task4))
+            .getEntities();
+    Assertions.assertThat(createdEntities).isNull();
   }
 
   private static TaskEntity createTask(String taskName, long id) {
@@ -247,41 +232,58 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
   /** Test that entity updates works well */
   @Test
-  void testUpdateEntities() {
+  protected void testUpdateEntities() {
     // allocate test driver
     polarisTestMetaStoreManager.testUpdateEntities();
   }
 
   /** Test that entity drop works well */
   @Test
-  void testDropEntities() {
+  protected void testDropEntities() {
     // allocate test driver
     polarisTestMetaStoreManager.testDropEntities();
   }
 
   /** Test that granting/revoking privileges works well */
   @Test
-  void testPrivileges() {
+  protected void testPrivileges() {
     // allocate test driver
     polarisTestMetaStoreManager.testPrivileges();
   }
 
   /** test entity rename */
   @Test
-  void testRename() {
+  protected void testRename() {
     // allocate test driver
     polarisTestMetaStoreManager.testRename();
   }
 
+  /** test entity lookup */
+  @Test
+  protected void testLookup() {
+    polarisTestMetaStoreManager.testLookup();
+  }
+
   /** Test the set of functions for the entity cache */
   @Test
-  void testEntityCache() {
+  protected void testEntityCache() {
     // allocate test driver
     polarisTestMetaStoreManager.testEntityCache();
   }
 
+  /** Test that attaching/detaching policies works well */
   @Test
-  void testLoadTasks() {
+  protected void testPolicyMapping() {
+    polarisTestMetaStoreManager.testPolicyMapping();
+  }
+
+  @Test
+  protected void testPolicyMappingCleanup() {
+    polarisTestMetaStoreManager.testPolicyMappingCleanup();
+  }
+
+  @Test
+  protected void testLoadTasks() {
     for (int i = 0; i < 20; i++) {
       polarisTestMetaStoreManager.createEntity(
           null, PolarisEntityType.TASK, PolarisEntitySubType.NULL_SUBTYPE, "task_" + i);
@@ -290,7 +292,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
     PolarisCallContext callCtx = polarisTestMetaStoreManager.polarisCallContext;
     List<PolarisBaseEntity> taskList =
-        metaStoreManager.loadTasks(callCtx, executorId, 5).getEntities();
+        metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(5)).getEntities();
     Assertions.assertThat(taskList)
         .isNotNull()
         .isNotEmpty()
@@ -310,7 +312,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
     // grab a second round of tasks. Assert that none of the original 5 are in the list
     List<PolarisBaseEntity> newTaskList =
-        metaStoreManager.loadTasks(callCtx, executorId, 5).getEntities();
+        metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(5)).getEntities();
     Assertions.assertThat(newTaskList)
         .isNotNull()
         .isNotEmpty()
@@ -324,7 +326,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
     // only 10 tasks are unassigned. Requesting 20, we should only receive those 10
     List<PolarisBaseEntity> lastTen =
-        metaStoreManager.loadTasks(callCtx, executorId, 20).getEntities();
+        metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(20)).getEntities();
 
     Assertions.assertThat(lastTen)
         .isNotNull()
@@ -338,7 +340,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
             .collect(Collectors.toSet());
 
     List<PolarisBaseEntity> emtpyList =
-        metaStoreManager.loadTasks(callCtx, executorId, 20).getEntities();
+        metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(20)).getEntities();
 
     Assertions.assertThat(emtpyList).isNotNull().isEmpty();
 
@@ -346,7 +348,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
     // all the tasks are unassigned. Fetch them all
     List<PolarisBaseEntity> allTasks =
-        metaStoreManager.loadTasks(callCtx, executorId, 20).getEntities();
+        metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(20)).getEntities();
 
     Assertions.assertThat(allTasks)
         .isNotNull()
@@ -361,13 +363,13 @@ public abstract class BasePolarisMetaStoreManagerTest {
     timeSource.add(Duration.ofMinutes(10));
 
     List<PolarisBaseEntity> finalList =
-        metaStoreManager.loadTasks(callCtx, executorId, 20).getEntities();
+        metaStoreManager.loadTasks(callCtx, executorId, PageToken.fromLimit(20)).getEntities();
 
     Assertions.assertThat(finalList).isNotNull().isEmpty();
   }
 
   @Test
-  void testLoadTasksInParallel() throws Exception {
+  protected void testLoadTasksInParallel() throws Exception {
     for (int i = 0; i < 100; i++) {
       polarisTestMetaStoreManager.createEntity(
           null, PolarisEntityType.TASK, PolarisEntitySubType.NULL_SUBTYPE, "task_" + i);
@@ -375,7 +377,6 @@ public abstract class BasePolarisMetaStoreManagerTest {
     PolarisMetaStoreManager metaStoreManager = polarisTestMetaStoreManager.polarisMetaStoreManager;
     PolarisCallContext callCtx = polarisTestMetaStoreManager.polarisCallContext;
     List<Future<Set<String>>> futureList = new ArrayList<>();
-    List<Set<String>> responses;
     ExecutorService executorService = Executors.newCachedThreadPool();
     try {
       for (int i = 0; i < 3; i++) {
@@ -390,7 +391,10 @@ public abstract class BasePolarisMetaStoreManagerTest {
                   do {
                     retry = false;
                     try {
-                      taskList = metaStoreManager.loadTasks(callCtx, executorId, 5).getEntities();
+                      taskList =
+                          metaStoreManager
+                              .loadTasks(callCtx, executorId, PageToken.fromLimit(5))
+                              .getEntities();
                       taskList.stream().map(PolarisBaseEntity::getName).forEach(taskNames::add);
                     } catch (RetryOnConcurrencyException e) {
                       retry = true;
@@ -399,21 +403,21 @@ public abstract class BasePolarisMetaStoreManagerTest {
                   return taskNames;
                 }));
       }
-      responses =
-          futureList.stream()
-              .map(
-                  f -> {
-                    try {
-                      return f.get();
-                    } catch (Exception e) {
-                      throw new RuntimeException(e);
-                    }
-                  })
-              .collect(Collectors.toList());
     } finally {
       executorService.shutdown();
-      Assertions.assertThat(executorService.awaitTermination(10, TimeUnit.MINUTES)).isTrue();
+      Assertions.assertThat(executorService.awaitTermination(30, TimeUnit.SECONDS)).isTrue();
     }
+    List<Set<String>> responses =
+        futureList.stream()
+            .map(
+                f -> {
+                  try {
+                    return f.get(30, TimeUnit.SECONDS);
+                  } catch (Exception e) {
+                    throw new RuntimeException(e);
+                  }
+                })
+            .collect(Collectors.toList());
     Assertions.assertThat(responses)
         .hasSize(3)
         .satisfies(l -> Assertions.assertThat(l.stream().flatMap(Set::stream)).hasSize(100));
@@ -428,7 +432,7 @@ public abstract class BasePolarisMetaStoreManagerTest {
 
   /** Test generateNewEntityId() function that generates unique ids by creating Tasks in parallel */
   @Test
-  void testCreateTasksInParallel() throws Exception {
+  protected void testCreateTasksInParallel() throws Exception {
     List<Future<List<Long>>> futureList = new ArrayList<>();
     Random rand = new Random();
     ExecutorService executorService = Executors.newCachedThreadPool();
