@@ -18,15 +18,16 @@
  */
 package org.apache.polaris.service.admin;
 
+import jakarta.enterprise.context.RequestScoped;
 import jakarta.inject.Inject;
 import jakarta.ws.rs.core.Response;
 import jakarta.ws.rs.core.SecurityContext;
 import java.util.List;
+import java.util.Locale;
 import org.apache.iceberg.catalog.Namespace;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.iceberg.exceptions.NotAuthorizedException;
 import org.apache.polaris.core.PolarisCallContext;
-import org.apache.polaris.core.PolarisConfiguration;
 import org.apache.polaris.core.admin.model.AddGrantRequest;
 import org.apache.polaris.core.admin.model.Catalog;
 import org.apache.polaris.core.admin.model.CatalogGrant;
@@ -37,11 +38,13 @@ import org.apache.polaris.core.admin.model.CreateCatalogRequest;
 import org.apache.polaris.core.admin.model.CreateCatalogRoleRequest;
 import org.apache.polaris.core.admin.model.CreatePrincipalRequest;
 import org.apache.polaris.core.admin.model.CreatePrincipalRoleRequest;
+import org.apache.polaris.core.admin.model.ExternalCatalog;
 import org.apache.polaris.core.admin.model.GrantCatalogRoleRequest;
 import org.apache.polaris.core.admin.model.GrantPrincipalRoleRequest;
 import org.apache.polaris.core.admin.model.GrantResource;
 import org.apache.polaris.core.admin.model.GrantResources;
 import org.apache.polaris.core.admin.model.NamespaceGrant;
+import org.apache.polaris.core.admin.model.PolicyGrant;
 import org.apache.polaris.core.admin.model.Principal;
 import org.apache.polaris.core.admin.model.PrincipalRole;
 import org.apache.polaris.core.admin.model.PrincipalRoles;
@@ -57,6 +60,7 @@ import org.apache.polaris.core.admin.model.UpdatePrincipalRoleRequest;
 import org.apache.polaris.core.admin.model.ViewGrant;
 import org.apache.polaris.core.auth.AuthenticatedPolarisPrincipal;
 import org.apache.polaris.core.auth.PolarisAuthorizer;
+import org.apache.polaris.core.config.FeatureConfiguration;
 import org.apache.polaris.core.context.CallContext;
 import org.apache.polaris.core.context.RealmContext;
 import org.apache.polaris.core.entity.CatalogEntity;
@@ -67,14 +71,19 @@ import org.apache.polaris.core.entity.PrincipalRoleEntity;
 import org.apache.polaris.core.persistence.MetaStoreManagerFactory;
 import org.apache.polaris.core.persistence.PolarisEntityManager;
 import org.apache.polaris.core.persistence.PolarisMetaStoreManager;
+import org.apache.polaris.core.secrets.UserSecretsManager;
+import org.apache.polaris.core.secrets.UserSecretsManagerFactory;
 import org.apache.polaris.service.admin.api.PolarisCatalogsApiService;
 import org.apache.polaris.service.admin.api.PolarisPrincipalRolesApiService;
 import org.apache.polaris.service.admin.api.PolarisPrincipalsApiService;
 import org.apache.polaris.service.config.RealmEntityManagerFactory;
+import org.apache.polaris.service.config.ReservedProperties;
+import org.apache.polaris.service.types.PolicyIdentifier;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /** Concrete implementation of the Polaris API services */
+@RequestScoped
 public class PolarisServiceImpl
     implements PolarisCatalogsApiService,
         PolarisPrincipalsApiService,
@@ -83,15 +92,26 @@ public class PolarisServiceImpl
   private final RealmEntityManagerFactory entityManagerFactory;
   private final PolarisAuthorizer polarisAuthorizer;
   private final MetaStoreManagerFactory metaStoreManagerFactory;
+  private final UserSecretsManagerFactory userSecretsManagerFactory;
+  private final CallContext callContext;
+  private final ReservedProperties reservedProperties;
 
   @Inject
   public PolarisServiceImpl(
       RealmEntityManagerFactory entityManagerFactory,
       MetaStoreManagerFactory metaStoreManagerFactory,
-      PolarisAuthorizer polarisAuthorizer) {
+      UserSecretsManagerFactory userSecretsManagerFactory,
+      PolarisAuthorizer polarisAuthorizer,
+      CallContext callContext,
+      ReservedProperties reservedProperties) {
     this.entityManagerFactory = entityManagerFactory;
     this.metaStoreManagerFactory = metaStoreManagerFactory;
+    this.userSecretsManagerFactory = userSecretsManagerFactory;
     this.polarisAuthorizer = polarisAuthorizer;
+    this.callContext = callContext;
+    this.reservedProperties = reservedProperties;
+    // FIXME: This is a hack to set the current context for downstream calls.
+    CallContext.setCurrentContext(callContext);
   }
 
   private PolarisAdminService newAdminService(
@@ -106,13 +126,16 @@ public class PolarisServiceImpl
         entityManagerFactory.getOrCreateEntityManager(realmContext);
     PolarisMetaStoreManager metaStoreManager =
         metaStoreManagerFactory.getOrCreateMetaStoreManager(realmContext);
+    UserSecretsManager userSecretsManager =
+        userSecretsManagerFactory.getOrCreateUserSecretsManager(realmContext);
     return new PolarisAdminService(
-        // FIXME remove call to CallContext.getCurrentContext()
-        CallContext.getCurrentContext(),
+        callContext,
         entityManager,
         metaStoreManager,
-        authenticatedPrincipal,
-        polarisAuthorizer);
+        userSecretsManager,
+        securityContext,
+        polarisAuthorizer,
+        reservedProperties);
   }
 
   /** From PolarisCatalogsApiService */
@@ -122,20 +145,20 @@ public class PolarisServiceImpl
     PolarisAdminService adminService = newAdminService(realmContext, securityContext);
     Catalog catalog = request.getCatalog();
     validateStorageConfig(catalog.getStorageConfigInfo());
-    Catalog newCatalog =
-        new CatalogEntity(adminService.createCatalog(CatalogEntity.fromCatalog(catalog)))
-            .asCatalog();
+    validateConnectionConfigInfo(catalog);
+    Catalog newCatalog = new CatalogEntity(adminService.createCatalog(request)).asCatalog();
     LOGGER.info("Created new catalog {}", newCatalog);
     return Response.status(Response.Status.CREATED).build();
   }
 
   private void validateStorageConfig(StorageConfigInfo storageConfigInfo) {
-    PolarisCallContext polarisCallContext = CallContext.getCurrentContext().getPolarisCallContext();
+    PolarisCallContext polarisCallContext = callContext.getPolarisCallContext();
     List<String> allowedStorageTypes =
         polarisCallContext
             .getConfigurationStore()
             .getConfiguration(
-                polarisCallContext, PolarisConfiguration.SUPPORTED_CATALOG_STORAGE_TYPES);
+                callContext.getRealmContext(),
+                FeatureConfiguration.SUPPORTED_CATALOG_STORAGE_TYPES);
     if (!allowedStorageTypes.contains(storageConfigInfo.getStorageType().name())) {
       LOGGER
           .atWarn()
@@ -146,6 +169,30 @@ public class PolarisServiceImpl
     }
   }
 
+  private void validateConnectionConfigInfo(Catalog catalog) {
+    if (catalog.getType() == Catalog.TypeEnum.EXTERNAL) {
+      if (catalog instanceof ExternalCatalog externalCatalog) {
+        if (externalCatalog.getConnectionConfigInfo() != null) {
+          String connectionType =
+              externalCatalog.getConnectionConfigInfo().getConnectionType().name();
+          List<String> supportedConnectionTypes =
+              callContext
+                  .getPolarisCallContext()
+                  .getConfigurationStore()
+                  .getConfiguration(
+                      callContext.getRealmContext(),
+                      FeatureConfiguration.SUPPORTED_CATALOG_CONNECTION_TYPES)
+                  .stream()
+                  .map(s -> s.toUpperCase(Locale.ROOT))
+                  .toList();
+          if (!supportedConnectionTypes.contains(connectionType)) {
+            throw new IllegalStateException("Unsupported connection type: " + connectionType);
+          }
+        }
+      }
+    }
+  }
+
   /** From PolarisCatalogsApiService */
   @Override
   public Response deleteCatalog(
@@ -196,7 +243,13 @@ public class PolarisServiceImpl
   public Response createPrincipal(
       CreatePrincipalRequest request, RealmContext realmContext, SecurityContext securityContext) {
     PolarisAdminService adminService = newAdminService(realmContext, securityContext);
-    PrincipalEntity principal = PrincipalEntity.fromPrincipal(request.getPrincipal());
+    PrincipalEntity principal =
+        new PrincipalEntity.Builder()
+            .setName(request.getPrincipal().getName())
+            .setClientId(request.getPrincipal().getClientId())
+            .setProperties(
+                reservedProperties.removeReservedProperties(request.getPrincipal().getProperties()))
+            .build();
     if (Boolean.TRUE.equals(request.getCredentialRotationRequired())) {
       principal =
           new PrincipalEntity.Builder(principal).setCredentialRotationRequiredState().build();
@@ -264,11 +317,15 @@ public class PolarisServiceImpl
       RealmContext realmContext,
       SecurityContext securityContext) {
     PolarisAdminService adminService = newAdminService(realmContext, securityContext);
+    PrincipalRoleEntity entity =
+        new PrincipalRoleEntity.Builder()
+            .setName(request.getPrincipalRole().getName())
+            .setProperties(
+                reservedProperties.removeReservedProperties(
+                    request.getPrincipalRole().getProperties()))
+            .build();
     PrincipalRole newPrincipalRole =
-        new PrincipalRoleEntity(
-                adminService.createPrincipalRole(
-                    PrincipalRoleEntity.fromPrincipalRole(request.getPrincipalRole())))
-            .asPrincipalRole();
+        new PrincipalRoleEntity(adminService.createPrincipalRole(entity)).asPrincipalRole();
     LOGGER.info("Created new principalRole {}", newPrincipalRole);
     return Response.status(Response.Status.CREATED).build();
   }
@@ -325,11 +382,15 @@ public class PolarisServiceImpl
       RealmContext realmContext,
       SecurityContext securityContext) {
     PolarisAdminService adminService = newAdminService(realmContext, securityContext);
+    CatalogRoleEntity entity =
+        new CatalogRoleEntity.Builder()
+            .setName(request.getCatalogRole().getName())
+            .setProperties(
+                reservedProperties.removeReservedProperties(
+                    request.getCatalogRole().getProperties()))
+            .build();
     CatalogRole newCatalogRole =
-        new CatalogRoleEntity(
-                adminService.createCatalogRole(
-                    catalogName, CatalogRoleEntity.fromCatalogRole(request.getCatalogRole())))
-            .asCatalogRole();
+        new CatalogRoleEntity(adminService.createCatalogRole(catalogName, entity)).asCatalogRole();
     LOGGER.info("Created new catalogRole {}", newCatalogRole);
     return Response.status(Response.Status.CREATED).build();
   }
@@ -519,8 +580,8 @@ public class PolarisServiceImpl
         catalogName);
     PolarisAdminService adminService = newAdminService(realmContext, securityContext);
     switch (grantRequest.getGrant()) {
-        // The per-securable-type Privilege enums must be exact String match for a subset of all
-        // PolarisPrivilege values.
+      // The per-securable-type Privilege enums must be exact String match for a subset of all
+      // PolarisPrivilege values.
       case ViewGrant viewGrant:
         {
           PolarisPrivilege privilege =
@@ -563,6 +624,19 @@ public class PolarisServiceImpl
           adminService.grantPrivilegeOnCatalogToRole(catalogName, catalogRoleName, privilege);
           break;
         }
+      case PolicyGrant policyGrant:
+        {
+          PolarisPrivilege privilege =
+              PolarisPrivilege.valueOf(policyGrant.getPrivilege().toString());
+          String policyName = policyGrant.getPolicyName();
+          String[] namespaceParts = policyGrant.getNamespace().toArray(new String[0]);
+          adminService.grantPrivilegeOnPolicyToRole(
+              catalogName,
+              catalogRoleName,
+              new PolicyIdentifier(Namespace.of(namespaceParts), policyName),
+              privilege);
+          break;
+        }
       default:
         LOGGER
             .atWarn()
@@ -595,8 +669,8 @@ public class PolarisServiceImpl
 
     PolarisAdminService adminService = newAdminService(realmContext, securityContext);
     switch (grantRequest.getGrant()) {
-        // The per-securable-type Privilege enums must be exact String match for a subset of all
-        // PolarisPrivilege values.
+      // The per-securable-type Privilege enums must be exact String match for a subset of all
+      // PolarisPrivilege values.
       case ViewGrant viewGrant:
         {
           PolarisPrivilege privilege =
@@ -639,6 +713,19 @@ public class PolarisServiceImpl
           adminService.revokePrivilegeOnCatalogFromRole(catalogName, catalogRoleName, privilege);
           break;
         }
+      case PolicyGrant policyGrant:
+        {
+          PolarisPrivilege privilege =
+              PolarisPrivilege.valueOf(policyGrant.getPrivilege().toString());
+          String policyName = policyGrant.getPolicyName();
+          String[] namespaceParts = policyGrant.getNamespace().toArray(new String[0]);
+          adminService.revokePrivilegeOnPolicyFromRole(
+              catalogName,
+              catalogRoleName,
+              new PolicyIdentifier(Namespace.of(namespaceParts), policyName),
+              privilege);
+          break;
+        }
       default:
         LOGGER
             .atWarn()
